@@ -14,7 +14,6 @@ from .utils.compat import weakref
 
 # THIRD-PARTY
 import numpy as np
-from astropy.extern.six.moves import xrange
 
 # LOCAL
 from . import great_circle_arc
@@ -68,7 +67,7 @@ class Graph:
         def __repr__(self):
             return "Node(%s %d)" % (str(self._point), len(self._edges))
 
-        def equals(self, other, thres=2e-8):
+        def equals(self, other, thresh=1.e-9):
             """
             Returns `True` if the location of this and the *other*
             `~Graph.Node` are the same.
@@ -84,7 +83,8 @@ class Graph:
                 empirical test cases. Relative threshold based on
                 the actual sizes of polygons is not implemented.
             """
-            return np.array_equal(self._point, other._point)
+            return great_circle_arc.same_point(self._point, other._point,
+                                               tol=thresh)
 
 
     class Edge:
@@ -476,12 +476,8 @@ class Graph:
         -------
         points : Nx3 array of (*x*, *y*, *z*) points
             This is a list of points outlining the union of the
-            polygons that were given to the constructor.  If the
-            original polygons are disjunct or contain holes, cut lines
-            will be included in the output.
+            polygons that were given to the constructor.
         """
-        self._remove_cut_lines()
-        self._sanity_check("union - remove cut lines")
         self._find_all_intersections()
         self._sanity_check("union - find all intersections")
         self._remove_interior_edges()
@@ -503,22 +499,29 @@ class Graph:
         -------
         points : Nx3 array of (*x*, *y*, *z*) points
             This is a list of points outlining the intersection of the
-            polygons that were given to the constructor.  If the
-            resulting polygons are disjunct or contain holes, cut lines
-            will be included in the output.
+            polygons that were given to the constructor.
         """
-        self._remove_cut_lines()
-        self._sanity_check("intersection - remove cut lines")
         self._find_all_intersections()
         self._sanity_check("intersection - find all intersections")
         self._remove_exterior_edges()
         self._sanity_check("intersection - remove exterior edges")
         self._remove_cut_lines()
-        self._sanity_check("intersection - remove cut lines [2]")
+        self._sanity_check("intersection - remove cut lines")
         self._remove_orphaned_nodes()
         self._sanity_check("intersection - remove orphan nodes", True)
         return self._trace()
 
+    def disjoint_polygons(self):
+        """
+        Convert a graph containing cut lines into a list of disjoint polygons
+        """
+        if self._remove_cut_lines():
+            self._sanity_check("disjoint - remove cut lines")
+            polygons = self._trace_polygons()
+        else:
+            polygons = list(self._source_polygons)
+        return polygons
+
     def _remove_cut_lines(self):
         """
         Removes any cutlines that may already have existed in the
@@ -557,14 +560,20 @@ class Graph:
         #     /                    \
         #
 
-        edges = list(self._edges)
-        for i in xrange(len(edges)):
-            AB = edges[i]
-            if AB not in self._edges:
-                continue
-            A, B = AB._nodes
+        cut_lines = []
+        changed = False
+
+        for edge in self._edges:
+            A, B = edge._nodes
             if len(A._edges) == 3 and len(B._edges) == 3:
-                self._remove_edge(AB)
+                cut_lines.append(edge)
+                changed = True
+
+        for edge in cut_lines:
+            if edge in self._edges:
+                self._remove_edge(edge)
+
+        return changed
 
     def _get_edge_points(self, edges):
         return (np.array([x._nodes[0]._point for x in edges]),
@@ -689,6 +698,7 @@ class Graph:
         Removes any nodes that are contained inside other polygons.
         What's left is the (possibly disjunct) outline.
         """
+        changed = False
         polygons = self._source_polygons
 
         for edge in self._edges:
@@ -707,14 +717,17 @@ class Graph:
         for edge in list(self._edges):
             if edge._count >= 1:
                 self._remove_edge(edge)
+                changed = True
 
-        self._remove_orphaned_nodes()
+        changed = self._remove_orphaned_nodes() or changed
+        return changed
 
     def _remove_exterior_edges(self):
         """
         Removes any edges that are not contained in all of the source
         polygons.  What's left is the (possibly disjunct) outline.
         """
+        changed = False
         polygons = self._source_polygons
 
         for edge in self._edges:
@@ -734,27 +747,33 @@ class Graph:
         for edge in list(self._edges):
             if edge._count < len(polygons):
                 self._remove_edge(edge)
+                changed = True
 
-        self._remove_orphaned_nodes()
+        changed = self._remove_orphaned_nodes() or changed
+        return changed
 
     def _remove_degenerate_edges(self):
         """
         Remove edges where both endpoints are the same point
         """
+        changed = False
         removals = []
         for edge in self._edges:
             if edge._nodes[0].equals(edge._nodes[1]):
                 removals.append(edge)
- 
+                changed = True
+
         for edge in removals:
             if edge in self._edges:
                 self._remove_edge(edge)
+        return changed
 
     def _remove_3ary_edges(self):
         """
         Remove edges between pairs of nodes that have odd numbers of
         edges.  This removes triangles that can't be traced.
         """
+        changed = False
         removals = []
         for edge in self._edges:
             nedges_a = len(edge._nodes[0]._edges)
@@ -762,32 +781,37 @@ class Graph:
             if (nedges_a % 2 == 1 and nedges_a >= 3 and
                 nedges_b % 2 == 1 and nedges_b >= 3):
                 removals.append(edge)
+                changed = True
 
         for edge in removals:
             if edge in self._edges:
                 self._remove_edge(edge)
+        return changed
 
     def _remove_orphaned_nodes(self):
         """
         Remove nodes with fewer than 2 edges.
         """
+        changed = False
         while True:
             removes = []
             for node in list(self._nodes):
                 if len(node._edges) < 2:
                     removes.append(node)
+                    changed = True
             if len(removes):
                 for node in removes:
                     if node in self._nodes:
                         self._remove_node(node)
             else:
                 break
+        return changed
 
-    def _trace(self):
+    def _trace_polygons(self):
         """
         Given a graph that has had cutlines removed and all
-        intersections found, traces it to find a resulting single
-        polygon.
+        intersections found, traces it to find a list of
+        disjoint polygons
         """
         from . import polygon as mpolygon
 
@@ -845,7 +869,16 @@ class Graph:
                     points.append(node._point)
                     break
 
-            polygon = mpolygon._SingleSphericalPolygon(points, auto_orient=True)
+            polygon = mpolygon._SingleSphericalPolygon(points)
             polygons.append(polygon)
 
-        return mpolygon.SphericalPolygon(polygons)
+        return polygons
+
+    def _trace(self):
+        """
+        Given a graph that has had cutlines removed and all
+        intersections found, traces it to find a resulting single
+        polygon.
+        """
+        from . import polygon as mpolygon
+        return mpolygon.SphericalPolygon(self._trace_polygons())
